/*****************************************************************

docbegin:

*  Name of source file:         j_ibr_get_jemx_arr_short.c

*  Version of source file:      0.00

*  Parent component:            None

*  Programmer:                  Niels J. Westergaard

*  Affiliation:                 Danish Space Research Institute

*  Contact address:             Juliane Maries Vej 30
*                               DK-2100 Copenhagen, Denmark
*                               Phone: +45 35325705  FAX: +45 35362475
*                               njw@dsri.dk

*  Purpose:                     Retrieve a specified JEMX array
*                               from the instrument data file in FITS

*  Origin date:                 991214

*  Update history:              0.00  991214  Copied from
*                                         JSIMGLgetJEMXArrShort.c
   
*  Type of element:             Function

*  Part of package:             j_ima_basic_recon

*  Name:                        j_ibr_get_jemx_arr_short

*  Calls:                       elementName
*                               parmGroupName
*                               array
*                               status

*  Returns:                     status

*  Required components:         DAL, PIL, RIL

docend:

*****************************************************************/

#include <isdc.h>

int j_ibr_get_jemx_arr_short(
         char *elementName,    /*  DS name of data element to get  */
         char *parmGroupName,  /*  File name of JEMX instr. data group  */
         short *array,         /*  short int array, returned    */
         int status)           /*  flag for status when called  */

{

   void *arrayPointer = NULL;
   char structureName[DAL_MAX_NAME_SIZE];

   int numAxes;

   long axes[DAL_MAX_ARRAY_DIMENSION];
   long startValues[2];
   long endValues[2];
   long numValues;

   dal_element *object;
   dal_element *element;
   dal_dataType DALtype;

   /*  No action if fault condition has been reported at time of call  */

   if( status != ISDC_OK ) return( status );

   /*  Define DOL of group  */

   strcpy( structureName, parmGroupName);
   strcat( structureName, "[GROUPING,1]" );

   /*  Open the group  */

   status = DALobjectOpen( structureName, &object, status );
   if( status != ISDC_OK ) return( status );


   /*  Find the correct element  */

   status = DALobjectFindElement( object, elementName, &element, status );
   if( status != ISDC_OK ) return( status );

   /*  Get the structure of the data array  */

   status = DALarrayGetStruct( element, &DALtype, &numAxes,
                        axes, status);
   if( status != ISDC_OK ) return( status );

   /*  Read the array data  */

   /*  DALtype = DAL_SHORT;  */
   DALtype = DT_UNKNOWN;
   startValues[0] = 1;
   startValues[1] = 1;
   endValues[0] = axes[0];
   endValues[1] = axes[1];

   arrayPointer = array;

   status = DALarrayGetSection( element, numAxes, startValues,
            endValues, &DALtype, &numValues, arrayPointer, status);
   if( status != ISDC_OK ) return( status );

   status = DALobjectClose( object, DAL_SAVE, status );
   return( status );

}
