/*****************************************************************

docbegin:

*  Name of file:                j_src_lc.h

*  Version of file:             0.01

*  Parent component:

*  Programmer:                  Stefan Larsson

*  Affiliation:                 Stockholm Observatory

*  Contact address:             Stockholm Observatory
*   			        SE-133 36 Saltsjobaden, Sweden
*                               Phone: +46 8-164464  FAX: +46 8-7174719
*                               stefan@astro.su.se

*  Purpose:                     Header file for 'j_src_lc'

*  Origin date:                 000511

*  Update history:              0.01   Partly copy of jextr5.h
   
*  Type of element:             Headerfile

*  Part of package:             j_src_extr

docend:

*****************************************************************/

#include <isdc.h>
#include <dal3gen.h>
#include <jemx.h>
#include <math.h>

#define J_EXTR_VER             "0.01" /*  Version number  */
#define J_EXTR_DIM_MASK_ARRAY     200  /*  Dimension of mask array  */
#define J_EXTR_SIZE_MASK_ARRAY  40000  /*  Size of mask array  */
#define J_EXTR_CLOSED_CELL          0
#define J_EXTR_OPEN_CELL            6

#define J_SRCEXTR_MAXSRC	   32  /*  Maximum number of sources */
#define J_SRCEXTR_MAXNCHANLC       32  /*  Maximum number of energy
					   channels (or bands) for 
					   light curves.  */
#define J_EXTR_ERROR_START  -0      /*  Error code offset  */
         /*  Local error codes:  */
#define J_EXTR_WRONG_JEMX_NUM  -1   /* Bad JEMX no.  */
#define J_EXTR_NOT_IN_SMODE    -2   /* Started not in ISDC single mode  */

/*
 *  Global variables for functions
 */

short maskPatternArray[J_EXTR_SIZE_MASK_ARRAY];
double sourcePosX[J_SRCEXTR_MAXSRC];
double sourcePosY[J_SRCEXTR_MAXSRC];
double sourceEffArea[J_SRCEXTR_MAXSRC];
double sourceEffAreaCor[J_SRCEXTR_MAXSRC][J_SRCEXTR_MAXSRC],
double sourceClosedAreaCor[J_SRCEXTR_MAXSRC][J_SRCEXTR_MAXSRC],

/*  Function Prototype Definitions  */

int j_src_lc_work(
         dal_element *ptr_inSWG,         /*  Pointer to input SWG  */
         JEMX_num jemxNum,               /*  Number of JEMX unit   */
         JEMX_type jemxType,             /*  Type of events        */
         int chatty,                     /*  Level of debugging output  */
         int status);                    /*  Status flag           */


int j_src_lc_params(
         dal_element **ptr_inSWG, /*  Pointer to SWG                */
         JEMX_num *jemxNum,       /*  JEM-X unit number             */
         JEMX_type *jemxType,     /*  Type of events (FULL or REST) */
         int *chatty,             /*  Level of debugging output     */
         int status );            /*  Status flag                   */
         
int j_src_lc_bin( dal_element *ptr_inSWG,         /* Pointer to input SWG */
	      JEMX_num jemxNum,                    /* Number of JEMX unit */
	      int numSources,     	           /* Number of sources  */
	      int nChanBins,                     /* Number of energy bins */
	      int chanLow[J_SRCEXTR_MAXNCHANLC], /* Lower energy limits */
	      int chanHigh[J_SRCEXTR_MAXNCHANLC],/* Upper energy limits */
	      double timeStep,		      /* Length of time bin */
	      OBTime OBTstart,                /* Start time for lightcurve */
	      OBTime OBTend,                  /* End time for lightcurve */
	      float *eventWeights	      /* Event weights */
	      int numEvents,		      /* Number of events */
	      int status )                    /* Status flag */

int j_ibr_get_jemx_arr_short( char *elementName,
                              char *parmGroupName,
                              short *array,                            
                              int status );


/*  Macros  */

