/*****************************************************************

docbegin:


*  Name of source file:         j_src_lc_main.c

*  Version of source file:      0.02

*  Parent component:            None

*  Programmer:                  Stefan Larsson

*  Affiliation:                 Stockholm Observatory

*  Contact address:             Stockholm Observatory
*   			        SE-133 36 Saltsjobaden, Sweden
*                               Phone: +46 8-164464  FAX: +46 8-7174719
*                               stefan@astro.su.se

*  Purpose:                     Extract spectra and light curves for individual
*                               sources in one science window of imaging data.

*  Origin date:                 000511

*  Update history:              0.01  000511 Adaption of test code jextr5.c
   
*  Type of element:             Executable

*  Part of package:             j_src_lc

*  Name:                        j_src_lc_main

*  Calls:                       j_src_lc_params
                                j_src_lc_work

*  Returns:

*  Required components:         DAL, PIL, RIL, DAL3GEN

docend:

Various remarks:

Flow of program:

Error codes:

   Basic offset:
   J_ERROR_CODE_START + J_SRCEEXTRACT_ERROR_START + J_EXTR_ERROR_START

*****************************************************************/

#include <j_src_lc.h>

int main (int argc, char *argv[])
{

  char dalErrorString[DAL_BIG_STRING]; /*  for the log of errors  */

  int status = ISDC_OK;
  int mode = 0;              /*  Running mode of this executable         */
  JEMX_num jemxNum;          /*  Number of JEM-X unit, should be 1 or 2  */
  JEMX_type jemxType;        /*  Type of events to be selected FULL,REST */
  int chatty;                /*  Level of debugging/monitoring output  */

  int i;                     /*  Loop variable for short loops   */

  dal_element  *ptr_inSWG = NULL;  /*  Pointer to SWG group  */

  LogFile fileRef;
  
  

  do {

    /* Use a once-only do/while loop to force program termination whenever
       status returned is not ISDC_OK  */
    
/*
 *
 *         Initialisation
 *
 */

    mode = CommonInit(argv[0], J_ISG_VER,argc,argv);
    if (mode != ISDC_SINGLE_MODE) {
      RILlogMessage(NULL, Error_2,
                "Not started in ISDC_SINGLE_MODE, mode = %d", mode);
      status = J_ERROR_CODE_START + J_SRCEEXTRACT_ERROR_START
                                  + J_EXTR_ERROR_START + J_EXTR_NOT_IN_SMODE;
      continue;
    }


  /*
   *      Initiate output log file
   */

   status = RILinit( &fileRef, NULL, OUT_ALL, argv[0]);
   if( status != ISDC_OK ) continue;



/*
 *
 *       Get the parameters, get instrument data, and prepare access to
 *           data structures (open the connections)
 *
 */

    status = j_src_lc_params(&ptr_inSWG,
                    &jemxNum, &jemxType, &chatty, status);
    if (status != ISDC_OK) {
    	RILlogMessage(NULL, Error_2, 
    		"Problem getting parameters etc., status = %d\n", status);
      continue;
    }


/*
 *
 *     Go ahead and do the work 
 *
 *
 */

    status = j_src_lc_work(ptr_inSWG, jemxNum, jemxType,
                  chatty, status);
    if (status != ISDC_OK) {
    	RILlogMessage(NULL, Error_2, 
    		"Problem in the work function, status = %d\n", status);
        continue;
    }	
    
    
    RILlogMessage(NULL, Log_2,"Work completed: status is %d\n", status);

   } while (0);
	

   status = CommonCloseSWG(ptr_inSWG, status);
   if (status != ISDC_OK) {
       RILlogMessage(NULL, Error_2, "Failed to close the SWG");
   }

   if( status != ISDC_OK ) {
       RILlogMessage(NULL, Error_2,
          "Terminating with status = %d", status);
   }


/*
 *  Dump the DAL error stack - for debugging purposes
 */

/*
   while ( status != DAL_NO_MORE_MESSAGES){
  	status = DALerrorReadStack(dalErrorString, status);
  	RILlogMessage(NULL,Error_2,"DAL error stack: %s .", dalErrorString);
   } 
   status = ISDC_OK;
*/
  
   CommonExit(status);

}


